
(* for printing architecture (-arch) and translation of probes (Compose.instance) *)
(* THIS STRUCTURE IS NOW SUPERSEDED BY JSON OUTPUT, EXCEPT FOR instance, SEE HOW TO REIMPLEMENT IT *)

structure Arch = struct

open OutPrint

datatype arch =
	 Leaf of string * (string * string) list * string list  (* process instance, with shvar map (for probes) and states *)
       | Node of string * arch list                                     (* component, with attached instances *)

val ATREE = ref ([] : arch list);

(* s = name, r = shared vars map, stl = state list *)
fun push_leaf s r stl =
    ATREE := Leaf (s,r,stl) :: !ATREE;

(* n = name, a = arity *)
fun push_node n a =
    let fun pop 0 l = l
	  | pop n l = let val h::t = !ATREE in ATREE := t; pop (n-1) (h::l) end
    in ATREE := Node (n, pop a []) :: !ATREE
    end;

val ANON = ref 0;
fun anon () =
    (ANON := 1 + !ANON; "_ANON_"^Int.toString (!ANON));

fun push_anon n a =
    let fun pop 0 l = l
	  | pop n l = let val h::t = !ATREE in ATREE := t; pop (n-1) (h::l) end
    in ATREE := Node (n,pop a []) :: !ATREE
    end;

fun instance path m =
    let fun find [] (Node (n,sl)) = (n,[],[])
	  | find (h::t) (Node (n,sl)) = find t (List.nth (sl,h-1))
	  | find _ (Leaf (n,al,stl)) = (n,al,stl)
	val SOME m' = List.find (fn Node (x,_) => x=m) (!ATREE) 
    in find path m'
    end;

(* for -arch, bottom up, as before *)
fun print_arch_bu () =
    if !ARCH then
	let fun name (Node (n,_)) = n | name (Leaf (n,_,_)) = n
	    fun prn (Node (n,sl)) =
		(List.app prn sl;
		 print n; List.app (fn s => (print " "; print (name s))) sl; print "\n")
	      | prn (Leaf (s,_,_)) =
		(print s; print "\n")
	in print "Component hierarchy:\n";
	   prn (hd (!ATREE));
	   print "\n";
	   flush ()
	end
    else
	();

(* top down *)
fun print_arch_td () =
    if !ARCH then
	let fun newline n = if n<0 then () else (print "\n"; repeat n print " ")
	    fun prn i (Node (n,sl)) = (newline i; print n; List.app (prn (i+2)) sl)
	      | prn i (Leaf (s,_,_)) = (newline i; print s)
	in print "Component hierarchy:\n";
	   prn 0 (hd (!ATREE));
	   print "\n";
	   flush ()
	end
    else
	();

end;




